@@ -166,7 +166,7 @@ class PropertyStats(object):
             return 0
 
         if weights is None:
-            return stats.skew(data_lst)
+            return stats.skew(np.array(data_lst))
         else:
             # Compute the mean
             mean = PropertyStats.mean(data_lst, weights)
@@ -244,12 +244,12 @@ class PropertyStats(object):
             mode
         """
         if weights is None:
-            return stats.mode(data_lst).mode[0]
+            return stats.mode(data_lst)[0]
         else:
             # Find the entry(s) with the largest weight
             data_lst = np.array(data_lst)
             weights = np.array(weights)
-            most_freq = np.isclose(weights, weights.max())
+            most_freq = np.isclose(weights, max(weights))
 
             # Return the minimum of the most-frequent entries
             return data_lst[most_freq].min()
@@ -346,60 +346,4 @@ class PropertyStats(object):
         """
         _ = weights
         q = float(q)
-        return np.quantile(data_lst, q=q)
-
-# class UnionFeaturizer(BaseFeaturizer, ABC):
-#
-#     def __init__(spath, comp, couple_data, couple=2, stats=("mean",)):
-#         super().__init__()
-#         spath.couple = couple
-#         spath.comp = comp
-#         spath.couple_data = couple_data
-#         spath.stats = stats
-#         # Initialize stats computer
-#
-#     def featurize(spath, comp_number=0):
-#         """
-#         Get elemental property attributes
-#
-#         Args:
-#             comp: Pymatgen composition object
-#
-#         Returns:
-#             all_attributes: Specified property statistics of features
-#             :param comp_number:
-#         """
-#         comp = spath.comp[comp_number]
-#         couple_data = spath.couple_data[comp_number].values
-#         all_attributes = []
-#
-#         # Get the element names and fractions
-#         elements, fractions = zip(*comp.element_composition.items())
-#         couple_data = np.reshape(couple_data, (spath.couple, -1), order="F")
-#         for elem_data_i in couple_data.T:
-#             for stat in spath.stats:
-#                 all_attributes.append(PropertyStats.calc_stat(elem_data_i, stat, fractions))
-#
-#         return all_attributes
-#
-#     def transform(spath, X=None, **kwargs):
-#         """Compute features for a list of inputs
-#
-#         Parameters
-#         ----------
-#         X
-#         **kwargs
-#         """
-#         number = list(range(len(spath.comp)))
-#         return spath.featurize_many(number, ignore_errors=True)
-#
-#     def feature_labels(spath):
-#         """
-#         Generate attribute names.
-#
-#         Returns:
-#             ([str]) attribute labels.
-#         """
-#         x_name = np.array(spath.couple_data.columns.values)[::spath.couple]
-#         x_name = [i.split("_")[0] for i in x_name]
-#         return x_name
+        return np.quantile(data_lst, q=q)
\ No newline at end of file
