@@ -6,24 +6,29 @@
 # @FileName: backforward.py
 # @Software: PyCharm
 
-"""forward_and_back feature elimination for feature ranking"""
+"""Forward_and_back feature elimination for feature ranking"""
 
 import copy
 from functools import partial
+from typing import List
 
 import numpy as np
 from sklearn.base import BaseEstimator, is_classifier
 from sklearn.base import MetaEstimatorMixin
 from sklearn.base import clone
 
+from sklearn.feature_selection import SelectorMixin
+
 from sklearn.metrics import check_scoring
 from sklearn.model_selection import check_cv
 from sklearn.model_selection._validation import _score, cross_val_score
+
 from sklearn.utils.metaestimators import if_delegate_has_method, _safe_split
 from sklearn.utils.validation import check_is_fitted, check_X_y, check_random_state
 
 from mgetool.tool import parallelize
-from .mutibase import MutiBase
+
+from featurebox.selection.mutibase import MutiBase
 
 
 def _baf_single_fit(train, test, baf, estimator, X, y, scorer, random_state):
@@ -38,33 +43,65 @@ def _baf_single_fit(train, test, baf, estimator, X, y, scorer, random_state):
 
 class BackForward(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
     """
+    BackForward method to selected features.
 
-    Parameters
-    ----------
-    estimator :
-        A supervised learning estimator with a ``_fit`` method that provides
+    estimator:
+        A supervised sklearn learning estimator with a ``_fit`` method that provides
         information about feature importance either through a ``coef_``
         attribute or through a ``feature_importances_`` attribute.
 
-    n_type_feature_to_select : int or None (default=None)
-        The number of feature to selection. If `None`, selection the features with best score .
+    n_type_feature_to_select: int or None (default=None)
+        The number of feature to selection. If `None`, selection the features with best score.
 
-
-    Attributes
-    ----------
-    n_feature_ : int
+    n_feature_: int
         The number of selected feature.
 
-    support_ : array of shape [n_feature]
+    support_: array of shape [n_feature]
         The mask of selected feature.
 
-    estimator_ : object
+    estimator_: object
         The external estimator _fit on the reduced dataset.
 
+    Examples:
+    -----------
+
+    >>> from sklearn.datasets import load_boston
+    >>> from sklearn.svm import SVR
+    >>> X,y = load_boston(return_X_y=True)
+    >>> svr= SVR()
+    >>> bf = BackForward(svr,primary_feature=4,  random_state=1)
+    >>> new_x = bf.fit_transform(X,y)
+    >>> bf.support_
+    array([False, False, False, False, False, False, False, False, False,
+           False,  True, False,  True])
     """
 
-    def __init__(self, estimator, n_type_feature_to_select=None, primary_feature=None, muti_grade=2, muti_index=None,
-                 must_index=None, tolerant=0.01, verbose=0, random_state=None):
+    def __init__(self, estimator: BaseEstimator, n_type_feature_to_select: int = None, primary_feature: int = None,
+                 muti_grade: int = 2, muti_index: List = None,
+                 must_index: List = None, tolerant: float = 0.01, verbose: int = 0, random_state: int = None):
+        """
+
+        Parameters
+        ----------
+        estimator:sklearn.estimator
+            sklearn.estimator
+        n_type_feature_to_select:int
+            force select number max
+        primary_feature:int
+             primary features to start loop, default n_features//2.
+        muti_grade: int
+            group number
+        muti_index:
+            group index
+        must_index:
+            must selection index
+        tolerant:
+            tolerant for rank compare.
+        verbose:int
+            print or not
+        random_state:int
+            random_state
+        """
         super().__init__(muti_grade=muti_grade, muti_index=muti_index, must_index=must_index)
         self.estimator = estimator
         self.n_type_feature_to_select = n_type_feature_to_select
@@ -148,7 +185,7 @@ class BackForward(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
                 for slice_, sub in zip(slice_all, slice10):
                     slice_.remove(sub)
                 test0 = [score(slices=i) for i in slice_all]
-                index = np.argmax(test0)
+                index = int(np.argmax(test0))
                 best0 = max(test0)
                 slice0 = slice_all[index]
                 # print(slice0, best0)
@@ -212,8 +249,7 @@ class BackForward(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
     @if_delegate_has_method(delegate='estimator')
     def predict(self, X):
-        """Reduce X to the selected feature and then Fit using the
-           underlying estimator.
+        """Reduce X to the selected feature and then using the underlying estimator to predict.
 
         Parameters
         ----------
@@ -230,8 +266,7 @@ class BackForward(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
     @if_delegate_has_method(delegate='estimator')
     def score(self, X, y):
-        """Reduce X to the selected feature and then return the score of the
-           underlying estimator.
+        """Reduce X to the selected feature and then return the score of the underlying estimator.
 
         Parameters
         ----------
@@ -249,16 +284,10 @@ class BackForward(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
         return self.support_
 
 
-class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
-    """Feature ranking with recursive feature elimination and cross-validated
-    selection of the best number of feature.
+class BackForwardCV(MetaEstimatorMixin, SelectorMixin,BaseEstimator):
+    """
+    BackForwardCV.
 
-    See glossary entry for :term:`cross-validation estimator`.
-
-    Read more in the :ref:`User Guide <baf>`.
-
-    Parameters
-    ----------
     estimator : object
         A supervised learning estimator with a ``_fit`` method that provides
         information about feature importance either through a ``coef_``
@@ -279,13 +308,6 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
         estimator is a classifier or if ``y`` is neither binary nor multiclass,
         :class:`sklearn.model_selection.KFold` is used.
 
-        Refer :ref:`User Guide <cross_validation>` for the various
-        cross-validation strategies that can be used here.
-
-        .. versionchanged:: 0.20
-            ``cv`` default value of None will change from 3-fold to 5-fold
-            in v0.22.
-
     scoring : string, callable or None, optional, (default=None)
         A string (see model evaluation documentation) or
         a scorer callable object / function with signature
@@ -300,8 +322,6 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
         for more details.
 
-    Attributes
-    ----------
     n_feature_ : int
         The number of selected feature with cross-validation.
 
@@ -310,11 +330,78 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
 
     estimator_ : object
         The external estimator _fit on the reduced dataset.
+
+
+    Examples:
+    -----------
+
+    >>> from sklearn.datasets import load_boston
+    >>> from sklearn.svm import SVR
+    >>> X,y = load_boston(return_X_y=True)
+    >>> svr= SVR()
+    >>> bf = BackForwardCV(svr,primary_feature=9,  random_state=1,cv=5)
+    >>> new_x = bf.fit_transform(X,y)
+    >>> bf.support_
+    array([False, False, False, False, False, False, False, False, False,
+           False,  True, False,  True])
+    >>> bf.score_
+    0.656494361629943
+
     """
 
-    def __init__(self, estimator, n_type_feature_to_select=None, primary_feature=None, muti_grade=2, muti_index=None,
-                 must_index=None, verbose=0, random_state=None,
-                 cv=5, scoring="r2", n_jobs=None):
+    def __init__(self, estimator: BaseEstimator, n_type_feature_to_select: int = None,
+                 primary_feature: int = None, muti_grade: int = 2, muti_index: List = None,
+                 must_index: List = None, verbose: int = 0, random_state: int = None,
+                 cv: int = 5, scoring: str = "r2", n_jobs: int = None, refit=False):
+        """
+
+        Parameters
+        ----------
+        estimator:sklearn.estimator
+            sklearn.estimator
+        n_type_feature_to_select:int
+            force select number max
+        primary_feature:int
+             expectation select number
+        muti_grade: int
+            group number
+        muti_index: list
+            group index
+        must_index: list
+            must selection index
+        random_state:int
+            random_state
+        cv : int, cross-validation generator or an iterable, optional
+            Determines the cross-validation splitting strategy.
+            Possible inputs for cv are:
+
+            - None, to use the default 3-fold cross-validation,
+            - integer, to specify the number of folds.
+            - CV splitter,
+            - An iterable yielding (train, test) splits as arrays of indices.
+
+            For integer/None inputs, if ``y`` is binary or multiclass,
+            :class:`sklearn.model_selection.StratifiedKFold` is used. If the
+            estimator is a classifier or if ``y`` is neither binary nor multiclass,
+            :class:`sklearn.model_selection.KFold` is used.
+
+        scoring : string, callable or None, optional, (default=None)
+            A string (see model evaluation documentation) or
+            a scorer callable object / function with signature
+            ``scorer(estimator, X, y)``.
+
+        verbose : int, (default=0)
+            Controls verbosity of output.
+
+        n_jobs : int or None, optional (default=None)
+            Number of cores to run in parallel while fitting across folds.
+            ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+            ``-1`` means using all processors.
+            for more details.
+        refit:
+            False
+        """
+
         self.estimator = estimator
         self.cv = cv
         self.scoring = scoring
@@ -327,6 +414,7 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
         self.must_index = must_index
         self.score_ = []
         self.random_state = random_state
+        self.refit = refit
         # spath.support_cv = ""
         # spath.score_cv = ""
         # spath.support_ = ""
@@ -335,8 +423,7 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
         # spath.n_feature_ = ""
 
     def fit(self, X, y, groups=None):
-        """Fit the baf model and automatically tune the number of selected
-           feature.
+        """Fit the baf model and automatically tune the number of selected feature.
 
         Parameters
         ----------
@@ -380,14 +467,14 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
         self.score_cv = scores
         self.score_ = best_score
         self.estimator_ = clone(self.estimator)
-        self.estimator_.fit(X[:, self.support_], y)
+        if self.refit:
+            self.estimator_.fit(X[:, self.support_], y)
         self.n_feature_ = np.count_nonzero(support)
         return self
 
     @if_delegate_has_method(delegate='estimator')
     def predict(self, X):
-        """Reduce X to the selected feature and then Fit using the
-           underlying estimator.
+        """Reduce X to the selected feature and then Fit using the underlying estimator.
 
         Parameters
         ----------
@@ -404,8 +491,7 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
 
     @if_delegate_has_method(delegate='estimator')
     def score(self, X, y):
-        """Reduce X to the selected feature and then return the score of the
-           underlying estimator.
+        """Reduce X to the selected feature and then return the score of the underlying estimator.
 
         Parameters
         ----------
@@ -421,3 +507,11 @@ class BackForwardCV(MetaEstimatorMixin, SelectorMixin):
     def _get_support_mask(self):
         check_is_fitted(self, 'support_')
         return self.support_
+
+# from sklearn.datasets import load_boston
+# from sklearn.svm import SVR
+# X,y = load_boston(return_X_y=True)
+# svr= SVR()
+# bf = BackForwardCV(svr, primary_feature=9,  random_state=1, cv =5)
+# new_x = bf.fit_transform(X,y)
+# bf.support_
