@@ -6,43 +6,67 @@
 # @FileName: exhaustion.py
 # @Software: PyCharm
 
-"""
-the estimator_ is the model with the best feature rather than all feature combination
-"""
-
 import warnings
 from functools import partial
 from itertools import combinations
+from typing import Tuple, List
 
 import numpy as np
 from sklearn.base import BaseEstimator
 from sklearn.base import MetaEstimatorMixin
 from sklearn.base import clone
-from sklearn.feature_selection.base import SelectorMixin
+from sklearn.feature_selection import SelectorMixin
+from sklearn.model_selection import cross_val_score
+
 from sklearn.utils.metaestimators import if_delegate_has_method
 from sklearn.utils.validation import check_is_fitted, check_X_y
 
 from mgetool.tool import parallelize
-from .mutibase import MutiBase
+from featurebox.selection.mutibase import MutiBase
 
 warnings.filterwarnings("ignore")
 
 
 class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
     """
-    the estimator_ is the model with the best feature rather than all feature combination
+    Exhaustion features combination.
+
+    The attribute ``estimator_`` is the model with the best feature rather than all feature combination.
+
+    >>> from sklearn.datasets import load_boston
+    >>> from sklearn.svm import SVR
+    >>> X,y = load_boston(return_X_y=True)
+    >>> svr= SVR()
+    >>> bf = Exhaustion(svr,n_select=(2,))
+    >>> new_x = bf.fit_transform(X,y)
+    >>> bf.support_
+    array([False, False, False, False, False, False, False, False, False,
+           False,  True, False,  True])
     """
 
-    def __init__(self, estimator, n_select=(2, 3, 4), muti_grade=2, muti_index=None, must_index=None, n_jobs=1,
-                 refit=False):
+    def __init__(self, estimator:BaseEstimator, n_select:Tuple=(2, 3, 4), muti_grade:int=2,
+                 muti_index:List=None, must_index:List=None, n_jobs:int=1,
+                 refit:bool=False,cv:int=5):
         """
 
-        :param estimator: and sklearn model or GridSearchCV
-        :param n_select: the n_select list,default,n_select=(3, 4)
-        :param muti_grade:muti_grade=2
-        :param muti_index:the range of muti_grade
-        :param must_index:the columns force to index
-        :param n_jobs:default=1
+        Parameters
+        ----------
+        estimator:
+            sklearn model or GridSearchCV
+        n_select:tuple
+            the n_select list,default,n_select=(3, 4)
+        muti_grade:list
+            binding_group size, calculate the correction between binding
+        muti_index:list
+            the range of muti_grade:[min,max)
+        must_index:list
+            the columns force to index
+        n_jobs:int
+            n_jobs
+        refit:bool
+            refit or not
+        cv:bool
+            if estimator is sklearn model, used cv, else pass
         """
         super().__init__(muti_grade=muti_grade, muti_index=muti_index, must_index=must_index)
         self.estimator = estimator
@@ -50,14 +74,14 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
         self.n_jobs = n_jobs
         self.n_select = [n_select, ] if isinstance(n_select, int) else n_select
         self.refit = refit
+        self.cv =cv
 
     @property
     def _estimator_type(self):
         return self.estimator._estimator_type
 
     def fit(self, X, y):
-        """Fit the baf model and then the underlying estimator on the selected
-           feature.
+        """Fit the baf model and then the underlying estimator on the selected feature.
 
         Parameters
         ----------
@@ -79,10 +103,13 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
                 slices = self.feature_unfold(slices)
                 data_x0 = x0[:, slices]
 
-                self.estimator.fit(data_x0, y0)
-
-                score0 = np.mean(self.estimator.best_score_)  # score_test
+                if hasattr(self.estimator,"best_score_"):
+                    self.estimator.fit(data_x0, y0)
+                    score0 = np.mean(self.estimator.best_score_)  # score_test
 
+                else:
+                    score0 = cross_val_score(self.estimator,data_x0,y0,cv=self.cv)
+                    score0 = np.mean(score0)
                 # print(slices, score0)
             return score0
 
@@ -104,7 +131,7 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
         feature_combination = [self.feature_unfold(_) for _ in slice_all]
         index = np.argmax(scores)
-        select_feature = feature_combination[index]
+        select_feature = feature_combination[int(index)]
         su = np.zeros(x.shape[1], dtype=np.bool)
         su[select_feature] = 1
         self.best_score_ = max(scores)
@@ -122,8 +149,7 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
     @if_delegate_has_method(delegate='estimator')
     def predict(self, X):
-        """Reduce X to the selected feature and then Fit using the
-           underlying estimator.
+        """Reduce X to the selected feature and then Fit using the underlying estimator.
 
         Parameters
         ----------
@@ -140,8 +166,7 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
     @if_delegate_has_method(delegate='estimator')
     def score(self, X, y):
-        """Reduce X to the selected feature and then return the score of the
-           underlying estimator.
+        """Reduce X to the selected feature and then return the score of the underlying estimator.
 
         Parameters
         ----------
